<?php

namespace Bydedi\Tracking\Google;

use Symfony\Component\HttpFoundation\Request;

/**
 * Class TagManager
 * @package Bydedi\Tracking\Google
 */
class TagManager
{
    /**
     * @var bool
     */
    protected static $isInit = false;

    /**
     * @var array|null $config
     */
    protected static $config = null;

    /**
     * @var array|null $events
     */
    protected static $events = null;

    /**
     * @param array $config
     */
    public static function init($config = array())
    {
        self::$config = $config;
    }

    /**
     * Check if it's initialized.
     * If not trigger a warning error
     */
    protected static function triggerErrorIfNotInit()
    {
        if (is_null(self::$config)) {
            trigger_error("Google Tag Manager n'est pas initialisé", E_USER_WARNING);
        }
    }

    protected static function assign(Request $request, $data_layer, $param)
    {
        $infos = static::common($request, $param);

        $tracker = "";

        if (!empty($infos)) {
            $data_layer = array_merge($infos[0], $data_layer);
            $tracker.= $infos[1];
        }

        $data_layer = '<script>var dataLayer = [];dataLayer.push(' . json_encode($data_layer) . ');</script>';

        if (isset($param['template'])) {
            $param['template']->reassign(self::$config['position_datalayer'], $data_layer . $tracker);
            $param['template']->reassign(self::$config['position'], $infos[2]);
        }
    }

    public static function common(Request $request, $param)
    {
        if (self::$isInit == true) {
            return false;
        }

        self::$isInit = true;

        $data_layer = array(
            "page" => $request->get('_route'),
        );

        $tracker = "
                <!-- Google Tag Manager -->
                <script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start':
                new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0],
                j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
                'https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
                })(window,document,'script','dataLayer','%s');</script>
                <!-- End Google Tag Manager -->
        ";

        $trackerNoJs = "
                <!-- Google Tag Manager (noscript) -->
                <noscript><iframe src=\"https://www.googletagmanager.com/ns.html?id=%s\"
                height=\"0\" width=\"0\" style=\"display:none;visibility:hidden\"></iframe></noscript>
                <!-- End Google Tag Manager (noscript) -->
        ";

        $data_layer = array_merge($data_layer, static::customer($request, $param, true));

        $tracker = sprintf($tracker, self::$config['id']);
        $trackerNoJs = sprintf($trackerNoJs, self::$config['id']);

        $events = (is_null(self::$events) == false) ? self::getPushedEventsJs() : '';

        return array($data_layer, $tracker, $events . $trackerNoJs);
    }

    public static function getVisites(Request $request, $param = array())
    {
        if (self::$isInit == true) {
            return false;
        }

        self::assign($request, array(), $param);
    }

    public static function home(Request $request, $param = array())
    {
        $data_layer = array();

        self::assign($request, $data_layer, $param);
    }

    public static function listing(Request $request, $param = array())
    {
        $data_layer = array(
            'ecommerce' => array(
                'currencyCode' => 'EUR',
                'impressions' => array(),
            ),
        );

        foreach ($param['products'] as $k => $product) {
            $item = array(
                'name' => $product['produit_nom_nom'],
                'id' => (string) $product['produit_id'],
                'price' => (string) $product['produit_prix_ttc'],
                'brand'=> $product['marque_nom'],
                'category' => '',
                'variant' => '',
                'list' => $param['type_listing'],
                'position' => (string) ($k+1),
            );
            $data_layer['ecommerce']['impressions'][] = $item;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function search(Request $request, $param = array())
    {
        $param['type_listing'] = 'recherche';
        static::listing($request, $param);
    }

    public static function productClick(Request $request, $param = array())
    {
        // TODO : a finir

        $data_layer = array(
            'event' => 'productClick',
            'ecommerce' => array(
                'click' => array(
                    'actionField' => array('list' => ''),
                    'products' => array(
                        'name' => '',
                        'id' => '',
                        'price' => '',
                        'brand' => '',
                        'category' => '',
                        'variant' => '',
                        'position' => '',
                    )
                )
            ),
        );
    }

    public static function product(Request $request, $param = array())
    {
        $data_layer = array(
            'ecommerce' => array(
                'detail' => array(
                    'actionField' => array(),
                    'products' => array(
                        array(
                            'name' => $param['produit_nom_nom'],
                            'id' => (string) $param['produit_id'],
                            'price' => (string) $param['produit_prix_ttc'],
                            'brand'=> $param['marque_nom'],
                            'category' => '',
                            'variant' => '',
                        ),
                    ),
                ),
            ),
        );

        static::assign($request, $data_layer, $param);
    }

    public static function customer(Request $request, $param = array(), $onlyDatalayer = false)
    {
        $data_layer = array(
            'client' => array(
                'id' => '',
                'email' => '',
                'type' => 'invite',
            )
        );

        $session = $request->getSession();

        if ($session->has('client_id')) {
            $client = new \Client($session->get('client_id'));
            $data_layer['client']['id'] = $client->client_id;
            $data_layer['client']['email'] = $client->client_email;
            $data_layer['client']['type'] = 'connecte';
        }

        if ($onlyDatalayer) {
            return $data_layer;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function addToCart(Request $request, $param = array())
    {
        // TODO : a finir

        $data_layer = array(
            'event' => 'addToCart',
            'ecommerce' => array(
                'currencyCode' => 'EUR',
                'add' => array(
                    'products' => array(
                        array(
                            'name' => '',
                            'id' => '',
                            'price' => '',
                            'brand' => '',
                            'category' => '',
                            'variant' => '',
                            'quantity' => ""
                        )
                    )
                )
            )
        );
    }

    public static function removeFromCart(Request $request, $param = array())
    {
        // TODO : a finir

        $data_layer = array(
            'event' => 'removeFromCart',
            'ecommerce' => array(
                'remove' => array(
                    'products' => [array(
                        'name' => '',
                        'id' => '',
                        'price' => '',
                        'brand' => '',
                        'category' => '',
                        'variant' => '',
                        'quantity' => ""
                    )]
                )
            )
        );
    }

    public static function checkout(Request $request, $param = array())
    {
        // TODO : a finir

        $data_layer = array(
            'ecommerce' => array(
                'checkout' => array(
                    'actionField' => array('step' => 1),
                    'products' => array()
                )
            ),
        );

        foreach ($param['extra']['produits'] as $product) {
            $item = [
                'name' => (string) $product['produit_nom'],
                'id' => (string) $product['produit_id'],
                'price' => (string) round($product['prix_ttc'], 2),
                'brand' => '',
                'category' => '',
                'variant' => '',
                'quantity' => (string) $product['qte'],
            ];

            $data_layer['ecommerce']['checkout']['products'][] = $item;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function checkoutOption(Request $request, $param = array())
    {
        $data_layer = array(
            'event' => 'checkoutOption',
            'ecommerce' => array(
                'checkout_option' => array(
                    'actionField' => array('step' => $param['step'], 'option' => $param['option'])
                ),
            )
        );

        static::assign($request, $data_layer, $param);
    }

    public static function purchase(Request $request, $param = array())
    {
        // TODO : a finir

        $data_layer = array(
            'ecommerce' => array(
                'purchase' => array(
                    'actionField' => array(
                        'id' => 'WEB-' . (string) $param['extra']['commande_id'],
                        'affiliation' => '',
                        'revenue' => (string) $param['extra']['commande_total_ttc'],
                        'tax' => (string) $param['extra']['commande_total_taxe'],
                        'shipping' => (string) $param['extra']['commande_port_ttc'],
                        'coupon' => ''
                    ),
                    'products' => array(),
                ),
            ),
        );

        foreach ($param['extra']['produits'] as $product) {
            $item = [
                'name' => (string) $product['produit_nom'],
                'id' => (string) $product['produit_id'],
                'price' => (string) $product['produit_prix_ttc'],
                'brand' => '',
                'category' => '',
                'variant' => '',
                'quantity' => (string) $product['produit_quantite'],
            ];

            $data_layer['ecommerce']['purchase']['products'][] = $item;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function pushEvent($node, $event, $category, $action)
    {
        self::$events[] = [
            'node' => $node,
            'event' => $event,
            'category' => $category,
            'action' => $action,
        ];
    }

    public static function getPushedEvents()
    {
        return self::$events;
    }

    public static function getPushedEventsJs()
    {
        $scripts = [];
        $mask = "$(document).on('click', '%s', function(){
                    dataLayer.push({
                        'event': '%s',
                        'eventcategory': '%s',
                        'eventaction': '%s',
                        'eventlabel': 'url: ' + window.location.href,
                        });
                    });";

        foreach (self::$events as $event) {
            $scripts[] = sprintf($mask, $event['node'], $event['event'], $event['category'], $event['action']);
        }

        return sprintf('<script>$(document).ready(function(){%s});</script>', implode('', $scripts));
    }
}
